local CC = ChrispyChars

local function CheckSlot(item) // this function deliberately errors when a freeslot does not exist
	if _G[item] == nil // this will error by itself for states and objects
		error() // this forces an error for sprites, which do actually return nil for some reason
	end
end

local function SafeFreeslot(...)
	local item = ...
	if item == nil
		return
	end
	
	if pcall(CheckSlot, item)
		print("\131NOTICE:\128 " .. item .. " was not allocated, as it already exists.")
	else
		freeslot(item)
	end
	
	SafeFreeslot(select(2, ...))
end

freeslot("SPR2_SLDE", "S_PLAY_SLIDE")


/*freeslot(
	"MT_PINSTASHIELD",
	"S_PINSTASHIELD",
	"S_PINSTASHIELD1A",
	"S_PINSTASHIELD1B",
	"S_PINSTASHIELD2A",
	"S_PINSTASHIELD2B",
	"S_PINSTASHIELD3A",
	"S_PINSTASHIELD3B",
	"S_PINSTASHIELD4A",
	"S_PINSTASHIELD4B",
	"S_PINSTASHIELD5A",
	"S_PINSTASHIELD5B",
	"S_PINSTASHIELD6A",
	"S_PINSTASHIELD6B",
	"SPR_PINT"
)

mobjinfo[MT_PINSTASHIELD] = {
	doomednum = -1,
	spawnhealth = 1,
	spawnstate = S_PINSTASHIELD,
	radius = 30*FRACUNIT,
	height = 48*FRACUNIT,
	flags = MF_NOGRAVITY|MF_NOBLOCKMAP|MF_NOCLIPHEIGHT|MF_MISSILE
}

states[S_PINSTASHIELD] = {SPR_NULL, 0, 1, A_CapeChase, 0, 0, S_PINSTASHIELD1A}
states[S_PINSTASHIELD1A] = {SPR_PINT, 0|FF_FULLBRIGHT, 1, A_CapeChase, 0, 0, S_PINSTASHIELD1B}
states[S_PINSTASHIELD1B] = {SPR_NULL, 0, 1, A_CapeChase, 0, 0, S_PINSTASHIELD2A}
states[S_PINSTASHIELD2A] = {SPR_PINT, 1|FF_FULLBRIGHT, 1, A_CapeChase, 0, 0, S_PINSTASHIELD2B}
states[S_PINSTASHIELD2B] = {SPR_NULL, 0, 1, A_CapeChase, 0, 0, S_PINSTASHIELD3A}
states[S_PINSTASHIELD3A] = {SPR_PINT, 2|FF_FULLBRIGHT, 1, A_CapeChase, 0, 0, S_PINSTASHIELD3B}
states[S_PINSTASHIELD3B] = {SPR_NULL, 0, 1, A_CapeChase, 0, 0, S_PINSTASHIELD4A}
states[S_PINSTASHIELD4A] = {SPR_PINT, 3|FF_FULLBRIGHT, 1, A_CapeChase, 0, 0, S_PINSTASHIELD4B}
states[S_PINSTASHIELD4B] = {SPR_NULL, 0, 1, A_CapeChase, 0, 0, S_PINSTASHIELD5A}
states[S_PINSTASHIELD5A] = {SPR_PINT, 4|FF_FULLBRIGHT, 1, A_CapeChase, 0, 0, S_PINSTASHIELD5B}
states[S_PINSTASHIELD5B] = {SPR_NULL, 0, 1, A_CapeChase, 0, 0, S_PINSTASHIELD6A}
states[S_PINSTASHIELD6A] = {SPR_PINT, 5|FF_FULLBRIGHT, 1, A_CapeChase, 0, 0, S_PINSTASHIELD6B}
states[S_PINSTASHIELD6B] = {SPR_NULL, 0, 1, A_CapeChase, 0, 0, S_NULL}*/

freeslot(
"S_PLAY_MID_FALL",
"S_DSDUST_SPAWN",
"S_DSDUST",
"SPR_DSRD"
)

states[S_DSDUST] = {SPR_DSRD,FF_TRANS60|FF_ANIMATE|A,12,nil,4,3,S_NULL}

states[S_DSDUST_SPAWN] = {SPR_DUST,FF_TRANS50,2,nil,0,0,S_DSDUST}

freeslot("MT_DSPSHOCKW", "S_DSP_SHOCKW1", "S_DSP_SHOCKW2", "SPR_DSPA")

mobjinfo[MT_DSPSHOCKW] = {
  mapnum = -1,
  spawnstate = S_DSP_SHOCKW1,
  flags = MF_NOGRAVITY|MF_MISSILE|MF_NOCLIPHEIGHT,
  spawnhealth = 1000,
  height = 18*FRACUNIT,
  radius = 15*FRACUNIT,
  speed = 25*FRACUNIT,
  reactiontime = 4*TICRATE/3,
  deathstate = 	S_NULL,
  seestate = S_DSP_SHOCKW1,
  seesound = sfx_none,
  deathsound = sfx_none
}

states[S_DSP_SHOCKW1] = {
 sprite = SPR_SHCK,
	frame = FF_FULLBRIGHT|FF_PAPERSPRITE,
	tics = -1,
	var1 = 1,
	var2 = 3,
  nextstate = S_DSP_SHOCKW1
}

states[S_PLAY_MID_FALL] = {
		sprite = SPR_PLAY,
		frame = SPR2_FLT_,
		var1 = 0,
		var2 = 0,
		tics = 4,
		nextstate = S_PLAY_FALL,
}

local DoFireDust = function(player, speed) // From the source code!
	local mo = player.mo
	local prandom = {}
	for i = 0, (leveltime%7) >> 1 // 1, 2, 3 or 4 particles
		local particle = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_SPINDUST)
		particle.state = S_SPINDUST_FIRE1

		particle.target = mo
		particle.destscale = (mo.destscale << 1)/3
		P_SetScale(particle, particle.destscale)
		if mo.eflags & MFE_VERTICALFLIP // readjust z position if needed
			particle.z = mo.z + mo.height - particle.height
		end
		prandom[0] = P_RandomFixed()<<2 // P_RandomByte()<<10
		prandom[1] = P_RandomRange(-30, 30) // P_RandomRange(-ANG30/FRACUNIT, ANG30/FRACUNIT)*FRACUNIT
		prandom[2] = P_RandomFixed()<<3 // P_RandomByte()<<11
		P_SetObjectMomZ(particle, speed/50 + prandom[0], false)
		P_InstaThrust(particle,
				player.drawangle + prandom[1]*ANG1,
				-FixedMul(speed/12 + FRACUNIT + prandom[2], mo.scale))
		P_TryMove(particle, particle.x + particle.momx, particle.y + particle.momy, true)
	end
end

local DSCpackCheck = function(player)
 if player.mo and player.mo.valid and player.mo.skin
  if player.mo.skin == "phoenix" or player.mo.skin == "nilly" or player.mo.skin == "michelle"
  or player.mo.skin == "caliburn" or player.mo.skin == "xalia" or player.mo.skin == "eblo"
   //we are using a DS character
   return true
  else

   return false
  end
 end
end

local SKIN = "phoenix"
local SKIN2 = "nilly"

local DoThokDust = function(player, speed) // From the source code!
	local mo = player.mo
	local prandom = {}
	for i = 0, (leveltime%7) >> 1 // 1, 2, 3 or 4 particles
		local particle = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_DUST)

		if mo.eflags & (MFE_TOUCHWATER|MFE_UNDERWATER)
			particle.state = S_INVISIBLE
		else
			//particle.state = S_DSDUST_SPAWN
		end

		particle.target = mo
		particle.destscale = (mo.destscale << 1)/4
		P_SetScale(particle, particle.destscale)
		particle.destscale = (mo.destscale << 1)
		if mo.eflags & MFE_VERTICALFLIP // readjust z position if needed
			particle.z = mo.z + mo.height - particle.height
		end
		prandom[0] = P_RandomFixed()<<2 // P_RandomByte()<<10
		prandom[1] = P_RandomRange(-15, 15) // P_RandomRange(-ANG30/FRACUNIT, ANG30/FRACUNIT)*FRACUNIT
		prandom[2] = P_RandomFixed()<<3 // P_RandomByte()<<11
		P_SetObjectMomZ(particle, speed/50 + prandom[0], false)
		P_InstaThrust(particle,
				player.drawangle + prandom[1]*ANG1,
				-FixedMul(speed/12 + FRACUNIT + prandom[2], mo.scale))
		P_TryMove(particle, particle.x + particle.momx, particle.y + particle.momy, true)
	end
end

local DoDSCWalkDust = function(player, speed) // From the source code!
	local mo = player.mo
	local prandom = {}
	if player.dsprevframe ~= player.mo.frame and (player.dsprevframe == 3 or player.dsprevframe == 7)
		local particle = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_DUST)

		if mo.eflags & (MFE_TOUCHWATER|MFE_UNDERWATER) // overrides fire version
			particle.state = S_INVISIBLE
		else
			//particle.state = S_DSDUST_SPAWN
		end

		particle.target = mo
		particle.destscale = (mo.destscale << 1)/4
		P_SetScale(particle, particle.destscale)
		particle.destscale = (mo.destscale << 1)
		if mo.eflags & MFE_VERTICALFLIP // readjust z position if needed
			particle.z = mo.z + mo.height - particle.height
		end
		prandom[0] = P_RandomFixed()<<2 // P_RandomByte()<<10
		prandom[1] = P_RandomRange(-15, 15) // P_RandomRange(-ANG30/FRACUNIT, ANG30/FRACUNIT)*FRACUNIT
		P_SetObjectMomZ(particle, speed/50 + prandom[0], false)
		P_InstaThrust(particle,
				player.drawangle + prandom[1]*ANG1,
				-FixedMul(speed/12 + FRACUNIT, mo.scale))
		P_TryMove(particle, particle.x + particle.momx, particle.y + particle.momy, true)
	end
end

local DoDSCRunDust = function(player, speed) // From the source code!
	local mo = player.mo
	local prandom = {}
	if leveltime%5 == 0
		local particle = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_DUST)

		if mo.eflags & (MFE_TOUCHWATER|MFE_UNDERWATER) // overrides fire version
			particle.state = S_INVISIBLE
		else
			//particle.state = S_DSDUST_SPAWN
		end

		particle.target = mo
		particle.destscale = (mo.destscale << 1)/4
		P_SetScale(particle, particle.destscale)
		particle.destscale = (mo.destscale << 1)
		if mo.eflags & MFE_VERTICALFLIP // readjust z position if needed
			particle.z = mo.z + mo.height - particle.height
		end
		prandom[0] = P_RandomFixed()<<2 // P_RandomByte()<<10
		prandom[1] = P_RandomRange(-15, 15) // P_RandomRange(-ANG30/FRACUNIT, ANG30/FRACUNIT)*FRACUNIT
		P_SetObjectMomZ(particle, speed/50 + prandom[0], false)
		P_InstaThrust(particle,
				player.drawangle + prandom[1]*ANG1,
				-FixedMul(speed/12 + FRACUNIT, mo.scale))
		P_TryMove(particle, particle.x + particle.momx, particle.y + particle.momy, true)
	end
end

local DoTaperThokTrail = function(mo, fuse, color, scale)

 local trail = P_SpawnMobj(mo.x,mo.y,mo.z,MT_THOK)
        trail.frame = TR_TRANS70
        trail.fuse = fuse
        trail.color = color
        trail.scale = mo.scale
        trail.destscale = trail.scale-scale
        return trail
end

local DoSpinDashDust2 = function(player, speed) // From the source code!
	local mo = player.mo
	local prandom = {}
	for i = 0, (leveltime%7) >> 1 // 1, 2, 3 or 4 particles
		local particle = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_SPINDUST)

		if mo.eflags & (MFE_TOUCHWATER|MFE_UNDERWATER) // overrides fire version
			particle.state = S_SPINDUST_BUBBLE1
		else
			particle.state = S_DSDUST_SPAWN
		end

		particle.target = mo
		particle.destscale = (mo.destscale << 1)/10
		P_SetScale(particle, particle.destscale)
		particle.destscale = (mo.destscale << 1)/8
		if mo.eflags & MFE_VERTICALFLIP // readjust z position if needed
			particle.z = mo.z + mo.height - particle.height
		end
		prandom[0] = P_RandomFixed()<<2 // P_RandomByte()<<10
		prandom[1] = P_RandomRange(-30, 30) // P_RandomRange(-ANG30/FRACUNIT, ANG30/FRACUNIT)*FRACUNIT
		prandom[2] = P_RandomFixed()<<3 // P_RandomByte()<<11
		P_SetObjectMomZ(particle, speed/50 + prandom[0], false)
		P_InstaThrust(particle,
				player.drawangle + prandom[1]*ANG1,
				-FixedMul(speed/12 + FRACUNIT + prandom[2], mo.scale))
		P_TryMove(particle, particle.x + particle.momx, particle.y + particle.momy, true)
	end
end

local function comet(player, color)
	local center = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z+player.mo.height/2, MT_THOK)
	center.target = player.mo
	center.scale = player.mo.scale/3
	if color
   center.color = color
 else
   center.color = player.mo.color
 end
	local radiusx = P_ReturnThrustX(player.mo, player.mo.angle, -player.mo.radius)
	local radiusy = P_ReturnThrustY(player.mo, player.mo.angle, -player.mo.radius)
	local factorx = P_ReturnThrustX(player.mo, player.mo.angle+ANGLE_90, FRACUNIT)
	local factory = P_ReturnThrustY(player.mo, player.mo.angle+ANGLE_90, FRACUNIT)
	local i = -1
	while (i < 2)
		local offseth = i*P_ReturnThrustX(player.mo, ANG20*leveltime, player.mo.radius)
		local offsetv = i*P_ReturnThrustY(player.mo, ANG20*leveltime, player.mo.radius)
		local x = player.mo.x + radiusx + FixedMul(offseth, factorx)
		local y = player.mo.y + radiusy + FixedMul(offseth, factory)
		local z = player.mo.z + (player.mo.height >> 1) + offsetv
		local trail = P_SpawnMobj(x, y, z, MT_THOK)
		trail.target = player.mo
		trail.scale = player.mo.scale/3
   if color
		trail.color = color
   else
   trail.color = player.mo.color
   end
		i = $+2
	end
end

addHook("ThinkFrame", function()
	for player in players.iterate
 if (player and player.mo and player.mo.valid) and (player.mo.skin == SKIN or player.mo.skin == SKIN2 or DSCpackCheck(player))
  
  if (player.pflags&PF_SLIDING) and (player.mo.sprite2 == SPR2_PAIN or player.mo.state == S_PLAY_PAIN) and player.mo.skin == SKIN
	player.mo.sprite2 = SPR2_STUN
  end
  
  if player.mo.state == S_PLAY_STUN
   player.mo.sprite2 = SPR2_PAIN
  end
  
  if (player.powers[pw_carry] == CR_MINECART and player.mo.state == S_PLAY_STND)
	player.mo.sprite2 = SPR2_LAND
  end
  
  if player.mo.state == S_PLAY_STND or player.mo.state == S_PLAY_WAIT or player.mo.state == S_PLAY_WALK or player.mo.state == S_PLAY_RUN or (player.mo.state == S_PLAY_DASH and (player.mo.momz*P_MobjFlip(player.mo) > FRACUNIT*2))
			if not P_IsObjectOnGround(player.mo)
			and player.powers[pw_carry] != CR_ROLLOUT
			and player.powers[pw_carry] != CR_MINECART
			and player.powers[pw_carry] != CR_MACESWING
			and player.powers[pw_carry] != CR_ZOOMTUBE
			and player.powers[pw_carry] != CR_ROPEHANG
			and player.powers[pw_carry] != CR_PLAYER
			and player.powers[pw_carry] != CR_NIGHTSMODE
				if player.mo.momz >= FRACUNIT
					player.mo.state = S_PLAY_SPRING
				//elseif player.mo.state == S_PLAY_RUN
				//	player.mo.state = S_PLAY_MID_FALL
				else
					player.mo.state = S_PLAY_FALL
				end
       if not player.dschargestate and not player.ya_walljump //and player.dschargestate == 2)
				player.pflags = $|PF_JUMPED|PF_NOJUMPDAMAGE & ~(PF_THOKKED|PF_STARTJUMP|PF_SPINNING)
       end
		end
	end
 else
  return
 end
 end
end)

addHook("PlayerThink", function(player)
    if not(player.mo and player.mo.valid) then return end     

	if not (player.mo.skin == SKIN or player.mo.skin == SKIN2 or DSCpackCheck(player)) then return end
	if player.mo.skin == SKIN then return end //temp
			
		if not P_IsObjectOnGround(player.mo) 
		and (player.mo.momz*P_MobjFlip(player.mo) <= FRACUNIT and player.mo.state == S_PLAY_SPRING
		or player.mo.momz*P_MobjFlip(player.mo) <= FRACUNIT and player.mo.state == S_PLAY_JUMP
		and (player.mo.skin == "phoenix" and not (player.pflags & PF_JUMPDOWN) and player.mo.momz*P_MobjFlip(player.mo) <= -FRACUNIT or player.mo.skin ~= "phoenix")
		or player.mo.momz*P_MobjFlip(player.mo) <= 0 and player.mo.state == S_PHOENIX_YAY)
			player.mo.state = (player.mo.state == S_PLAY_JUMP and player.mo.skin == "phoenix") and S_PLAY_FALL or S_PLAY_MID_FALL
			player.panim = PA_FALL
		end

		if player.mo.state == S_PLAY_JUMP or ((player.pflags&PF_JUMPED) and not (player.pflags&PF_SPINNING)) or (player.mo.eflags&MFE_SPRUNG)
			player.accelstart = skins[player.mo.skin].accelstart*5/9 //skins[player.mo.skin].accelstart
			player.acceleration = (player.mun_dashstop or player.mo.state == S_PHOENIX_AIRDASH) and 58 or skins[player.mo.skin].acceleration
			player.thrustfactor = (player.speed > 15*FRACUNIT) and skins[player.mo.skin].thrustfactor * 12/10 or skins[player.mo.skin].thrustfactor * 15/10
		else
		
			player.thrustfactor = (player.speed > player.runspeed/4 and (player.dschargestate and player.dschargestate < 2 or not player.dschargestate or not player.yaN_dashmode) and not (player.mo.eflags&MFE_UNDERWATER)) and 5 or 5 //skins[player.mo.skin].thrustfactor
     //if not player.genphysics
			player.accelstart = skins[player.mo.skin].accelstart
			player.acceleration =  skins[player.mo.skin].acceleration
			
			/*if player.mo.skin == "phoenix"
				player.acceleration = (player.speed < 37*FRACUNIT and not (player.dschargestate or player.yaN_dashmode or (player.dashmode >= 3*TICRATE))) and 30 or 40
			else
				player.acceleration = (player.speed < 37*FRACUNIT and not (player.dschargestate or player.yaN_dashmode or (player.dashmode >= 3*TICRATE))) and skins[player.mo.skin].acceleration + 1 or skins[player.mo.skin].acceleration/28 * 40
			end
			
			if (player.mo.flags2&MF2_TWOD)
				player.acceleration = $*2
			end*/
		end
		
		
    if (P_IsObjectOnGround(player.mo)) 
		if player.mo.state == S_PLAY_DASH and (player.cmd.forwardmove or player.cmd.sidemove)
		and not player.powers[pw_super]
			DoThokDust(player, 12*FRACUNIT)
		elseif player.speed > FRACUNIT
				if (player.pflags&PF_SPINNING)
					//if player.mo.state == S_PLAY_ROLL
						P_SpawnSkidDust(player, 7*FRACUNIT, sfx_thok)
					//end
				elseif player.mo.state == S_PLAY_RUN or player.speed >= player.runspeed
					/*if player.mo.state == S_PLAY_WALK
						DoDSCWalkDust(player, player.speed)
					else*/
						DoDSCRunDust(player, player.speed/2)
					//end
				end
		end
		
		player.prevframe = player.mo.frame
    end
end)

-- Wall Check code by Buggie

rawset(_G, "P_IsClimbingValid",function(player, angle,sec)

	local platx, platy
	local glidesector
	local floorz, ceilingz
	local mo = player.mo
	local rover

	platx = P_ReturnThrustX(mo, angle, mo.radius + FixedMul(8*FRACUNIT, mo.scale))
	platy = P_ReturnThrustY(mo, angle, mo.radius + FixedMul(8*FRACUNIT, mo.scale))

	glidesector = R_PointInSubsector(mo.x + platx, mo.y + platy).sector
	if sec then glidesector = sec end
	
	floorz = glidesector.floorheight
	ceilingz = glidesector.ceilingheight

	if (glidesector ~= mo.subsector.sector)
	
		local floorclimb = false
		local topheight, bottomheight

		for rover in glidesector.ffloors()
		
			if (not (rover.flags & FF_EXISTS) or not (rover.flags & FF_BLOCKPLAYER))
				continue
			end

			topheight = rover.topheight
			bottomheight = rover.bottomheight

			floorclimb = true

			if (mo.eflags & MFE_VERTICALFLIP)
			
				if ((topheight < mo.z + mo.height) and ((mo.z + mo.height + mo.momz) < topheight))
					floorclimb = true
				end
				if (topheight < mo.z)
					floorclimb = false
				end
				if (bottomheight > mo.z + mo.height - FixedMul(16*FRACUNIT,mo.scale))
					floorclimb = false
				end
			else
			
				if ((bottomheight > mo.z) and ((mo.z - mo.momz) > bottomheight))
					floorclimb = true
				end
				if (bottomheight > mo.z + mo.height)
					floorclimb = false
				end
				if (topheight < mo.z + FixedMul(16*FRACUNIT,mo.scale))
					floorclimb = false
				end
			end

			if (floorclimb)
				break
			end
		end

		if (mo.eflags & MFE_VERTICALFLIP)
		
			if ((floorz <= mo.z + mo.height)
				and ((mo.z + mo.height - mo.momz) <= floorz))
				floorclimb = true
			end

			if ((floorz > mo.z)
				and glidesector.floorpic == "F_SKY1")
				return false
			end

			if ((mo.z + mo.height - FixedMul(16*FRACUNIT,mo.scale) > ceilingz)
				or (mo.z + mo.height <= floorz))
				floorclimb = true
			end
		else
		
			if ((ceilingz >= mo.z)
				and ((mo.z - mo.momz) >= ceilingz))
				floorclimb = true
			end

			if ((ceilingz < mo.z+mo.height)
				and glidesector.ceilingpic == "F_SKY1")
				return false
			end

			if ((mo.z + FixedMul(16*FRACUNIT,mo.scale) < floorz)
				or (mo.z >= ceilingz))
				floorclimb = true
			end
		end

		if (not floorclimb)
			return false
		end

		return true
	end

	return true
end)

rawset(_G,"PTR_GlideClimbTraverse",function(slidemo,li)
	if not li then return end
	local checkline = li
	local rover
	local topheight, bottomheight
	local fofline = false
	local checksector = (li.backsector and not P_PointOnLineSide(slidemo.x, slidemo.y, li)) and li.backsector or li.frontsector

	if (checksector.ffloors)
		for rover in checksector.ffloors()
			if (not(rover.flags & FF_EXISTS) or not(rover.flags & FF_BLOCKPLAYER) or ((rover.flags & FF_BUSTUP) and (slidemo.player.charflags & SF_CANBUSTWALLS)))
				continue
			end

			topheight = rover.topheight
			bottomheight = rover.bottomheight

			if (topheight < slidemo.z)
				continue
			end

			if (bottomheight > slidemo.z + slidemo.height)
				continue
			end

			// Got this far, so I guess it's climbable. // TODO: Climbing check, also, better method to do this?
			if (rover.master.flags & ML_TFERLINE)
				local linenum = checksector.lines[0]
				checkline = rover.master.frontsector.lines[0] + linenum
				fofline = true
			end

			break
		end
	end

	// see about climbing on the wall
	if (not(checkline.flags & ML_NOCLIMB) and checkline.special ~= 41)
		local canclimb
		local climbangle, climbline
		local whichside = P_PointOnLineSide(slidemo.x, slidemo.y, li)

		climbangle = R_PointToAngle2(li.v1.x, li.v1.y, li.v2.x, li.v2.y)
		climbline = climbangle

		if (whichside) // on second side?
			climbline = $+ ANGLE_180
		end

		climbangle = $+ (ANGLE_90 * (whichside and -1 or 1))

		canclimb = (li.backsector and P_IsClimbingValid(slidemo.player,climbangle,checksector) or true)
		
		if (((not slidemo.player.climbing)
		or (slidemo.player.climbing == 1))
		and canclimb)
			return true,climbangle,checksector
		end
	end
	return false,0
end)


freeslot("SPR_WLHL","SPR_WLSP", "S_CLIMBWALL_HOLD","S_CLIMBWALL_SPIN")
states[S_CLIMBWALL_HOLD] = {SPR_WLHL,FF_ANIMATE|FF_PAPERSPRITE,18,nil,G,2,S_NULL}
states[S_CLIMBWALL_SPIN] = {SPR_WLSP,FF_ANIMATE|FF_PAPERSPRITE,10,nil,E,2,S_NULL}

freeslot("sfx_airdsh", "SPR_DSHR", "MT_DASH_RING", "S_DASH_RING")

//sfxinfo[sfx_pxjump].caption = "Jump"
sfxinfo[sfx_airdsh].caption = "Air Dash"

mobjinfo[MT_DASH_RING] = {
    doomednum = -1,
    spawnstate = S_DASH_RING,
    flags = MF_SCENERY|MF_NOCLIP|MF_NOGRAVITY
}
states[S_DASH_RING] = {
    sprite = SPR_DSHR,
    frame = A
}

addHook("MobjThinker", function(mo)
  local fusefade = {
    [0] = 0,
    [1] = FF_TRANS90,
    [2] = FF_TRANS80,
    [3] = FF_TRANS70,
    [4] = FF_TRANS60,
    [5] = FF_TRANS50,
    [6] = FF_TRANS40,
    [7] = FF_TRANS30,
    [8] = FF_TRANS20,
    [9] = FF_TRANS10
	}
  if mo and mo.valid
    if mo.fuse < 10
    and fusefade[mo.fuse/2]
      local fade = fusefade[mo.fuse/2]
      mo.frame = $|fade 
    end
  end
end,MT_DASH_RING)

freeslot(
	"SPR_REBO",
	"MT_REBOUND",
	"S_REBOUND",
	"sfx_airdsh",
	"sfx_bounc1",
	"sfx_bounc2"
)

/*COM_AddCommand("DSP_abilityswap", function(p)
	if not (p and p.valid and p.mo and p.mo.valid and p.mo.skin == "phoenix")
	CONS_Printf(p, "Nuh uh")
	return
	end
	if p.dspswap
	p.dspswap = 0
	p.charability = CA_JUMPTHOK
	CONS_Printf(p, "\x85"+"Phoenix now uses the Battery Dash.")
	else
	p.dspswap = 1
	p.charability = CA_DOUBLEJUMP
	CONS_Printf(p, "\x85"+"Phoenix now uses the Twirlwind.")
	end
end)*/